
/////////////////////////////////////////////////////////////////////////////
// WiFi MelderV2 mit 1 Eingangskontakt
// Firmware V1.3.0
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// rtcMemory Belegung
// 0-> Zyklus 
// 4-> Zustandsleep, letzer Zustad vor sleep Modus
// 8-> Periodenflag, Perdiodische Benachrichtigung Ja/Nein
// 12-> WifiConnf, Wifi Connection Fehlerzähler
// 16-> httpConnf, cloud Übertragungsfehler
// 20-> bew, Bewegungsrichtung beim letzten Conn. Fehler
// 24-> aufzaehler, Aufzähler beim letzten Conn. Fehler
// 28-> zuzaehler, Zuzähler beim letzten Conn. Fehler
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// LED Codes:
// 8x schnell blinken : Wifi Connect failed in OTA Phase
// 1x lang leuchten (ca 4 Sek) : OTA ready
// 3x lang blinken : Setup gestartet
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//zustandalt 1 = Tür AUF
//zustandalt 0 = Tür ZU
/////////////////////////////////////////////////////////////////////////////


//OTA Code (lib´s)
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
bool ota=0;
//END OTA Code

#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <EEPROM.h>
#include <ESP8266HTTPClient.h>
#include <Ticker.h>  //Ticker Library
#include "WEBset.h"  // Hier ist die HTML-Webseite als Array abgelegt
#include <ESP8266WiFi.h>
#include <TimeLib.h>
#include <Timezone.h>
#include "time_ntp.h"

#define onboardled 2 //GPIO2 ist On Board LED PIN bei ESP8266 12S, GPIO16 bei NODEMCU
#define reedpin 5 // Readkontakt ist an GPIO5 angeschlossen 
//#define dippin 13 // DIP Kontakt ist an GPIO13 angeschlossen
#define ffclr 4 // PIN zum FlipFlop CLR PIN

IPAddress apIP(192, 168, 2 , 117);  // IP-Adresse für Setup

//bool bNTPStarted=false;
bool zustandalt=0; // Zustand Reedkontakt alt
//bool zustandaltd=0; // Zustand DIP Schalter alt
//bool zustandalt2=0; // Zustand Reedkontakt alt zweite Phase
//bool periodenflag2=0; //Flag2 für periodische Benachrichtigung Zustand aus Eeprom 
bool zustandneu=0; // Zustand Reedkontakt neu
bool bewflag=0;// Hier wird gesetzt ob die Bewegung durch ein auf oder zu getriggert wurde

unsigned int periodenflag1[2]; //Flag1 für periodische Benachrichtigung verändert innerhalb sketch 
unsigned int zustandsleep[2]; //Flag welches in der rtc usermemory gespeichert wird, Es wird der Zustand (Auf oder Zu gespeichert)
unsigned int wificonnf[2]; // Wifi Verbindungsfehlerzähler 
unsigned int httpconnf[2]; // http Verbindungsfehlerzähler 
unsigned int wificonnfz[2]; // Wifi Verbindungsfehlerzähler Zwischenspeicher
unsigned int httpconnfz[2]; // http Verbindungsfehlerzähler Zwischenspeicher
unsigned int Zyklus[2]; // Zykluszähler

uint32_t aufzaehler=0; // Zuzähler für Auf 
uint32_t zuzaehler=0; // Zuzähler für ZU 
uint32_t aufzaehlerz=0; // Zuzähler für Auf Zwischenspeicher
uint32_t zuzaehlerz=0; // Zuzähler für ZU Zwischenspeicher

uint ADCKal34, ADCKal24;

char periodstr[4]=""; // periodenflag Daten welche von EEPROM kommen (Ja/NEIN)
char periodenabstandstr[3]=" "; //Zeitabstand in Sek. zwischen den periodischen Benachrichtigungen aus EEPROM

//int sr=0;
//int wh=0;
int periodenabstand=30; //Zeitabstand in Sek. zwischen den periodischen Benachrichtigungen 
int z = 0; //EEPROM Zeiger der Adresse zum auslesen
int prellzaehler =0;
int timeout = 0;
int httpCode =0;



//int laengessid =0; //ssid Länge wird hier abgelegt
//int laengepw =0;  //pw Länge wird hier abgelegt
String periodenabstandstring ="" ; //Zeitabstand in Sek. zwischen den periodischen Benachrichtigungen, String Variante zum Umwandeln in INT
String nachricht = ""; //Nachricht für Push Versand aus EEPROM
String nachricht2 =""; //Nachricht für Push Versand , Zusatz aus Sketch
String schlussz= "";// Letzter Zustand des reedkontaktes
String url = ""; //URL für Push aus EEPROM
String zustand=""; //Auf oder Zu , wird an Push Nachricht angehangen

char ssid[32] = "fhgdhfghd\0";
char passwort[64] = "gghdfgh\0";
const byte reed=reedpin;
ADC_MODE (ADC_VCC);           // ADC an das Stromversorgung
ESP8266WebServer server;
HTTPClient http;
//ESP8266WebServer httpServer(80);
Ticker zaehler;


extern "C" {
#include "user_interface.h"
}
#include "Setup.h" //Setup Sketch für WLAN Melder


//////////////////////////////////////////
// for time conversion
//Central European Time (Frankfurt, Paris)
//////////////////////////////////////////
TimeChangeRule CEST = {"CEST", Last, Sun, Mar, 2, 120};     //Central European Summer Time
TimeChangeRule CET = {"CET ", Last, Sun, Oct, 3, 60};       //Central European Standard Time
Timezone CE(CEST, CET);
 ///////////////////////////////////////////
// get UTC time referenced to 1970 by NTP
///////////////////////////////////////////
time_t getNTP_UTCTime1970() 
{ 
  //bNTPStarted=false;  // invalidate; time-lib functions crash, if not initalized poperly
  unsigned long t = getNTPTimestamp();
  
  if (t==0) return(0);

  // scale to 1970 
  // may look like back & forth with ntp code; wrote it to make needed conversions more clear
  return(t+946684800UL);
}

void setup() {
  pinMode(onboardled, OUTPUT); //GPIO2 ist On Board LED PIN bei ESP8266 12S, GPIO16 bei NODEMCU
  
  
  EEPROM.begin(250);  // EEPROM initialisieren mit 250 Byts
    // EEPROM Pointer Z zurücksetzen und Einstellungen aus EEPROM einlesen
  z = 0;
  LeseEeprom(ssid, sizeof(ssid));
  LeseEeprom(passwort, sizeof(passwort));
  LeseEepromStr(&url, 100);
  LeseEepromStr(&nachricht, 100);
  LeseEeprom(periodstr,4);
  LeseEeprom(periodenabstandstr,3);

    
  system_get_rst_info();  // Resetgrund einlesen
  const rst_info * resetInfo = ESP.getResetInfoPtr();  // Resetgrund einlesen
  ESP.rtcUserMemoryRead(0, Zyklus, sizeof(Zyklus));  // Zyklus Nummer einlesen
  //Wenn Reset aufgrunf Hardware Reset war(Batterie raus usw), Werte Zurücksetzen.
  if ((resetInfo->reason) == 6 and Zyklus[0]!=1 and Zyklus[0]!=2){
    Zyklus[0] = 0;         // Wenn Spannung OFF verschiedene Werte zurücksetzen und einige in rtcMemory schreiben
    wificonnf[0] = 0 ;
    httpconnf[0] = 0;
    wificonnfz[0] = 0 ;
    httpconnfz[0] = 0;
    aufzaehlerz = 0;
    zuzaehlerz = 0;
    ESP.rtcUserMemoryWrite(0, Zyklus, sizeof(Zyklus)); // Zyklus Nummer speichern
    ESP.rtcUserMemoryWrite(12, wificonnf, sizeof(wificonnf)); 
    ESP.rtcUserMemoryWrite(16, httpconnf, sizeof(httpconnf)); 
    ESP.rtcUserMemoryWrite(24, &aufzaehler, sizeof(aufzaehler));
    ESP.rtcUserMemoryWrite(28, &zuzaehler, sizeof(zuzaehler));
  }

  pinMode(reedpin, INPUT); // GPIO5 zum lesen des Reed Zustand
  //pinMode(dippin, INPUT); // GPIO3 zum lesen des DIP Zustandes
  pinMode(ffclr, OUTPUT); // GPIO zu Flip Flop R PIN
  zustandalt = digitalRead(reedpin); // reed Zustand einlesen
  zustandalt = !zustandalt; // invertieren für Hardwareversion mit nur 1 reedpin
  

Serial.begin(115200);

//Serial.printf("\n zustandalt= %i resetinfo= %i zyklus = %i \n",zustandalt, resetInfo->reason, Zyklus[0]);


 

 //------------------------------------ Prüfen ob Tx und Rx verbunden sind --------------
//  pinMode(1, OUTPUT); // Tx
//  pinMode(3, INPUT_PULLUP); // Rx
//  digitalWrite(1, 1);
//  delayMicroseconds(2);
//  if (digitalRead(3)==1){ // Prüfen ob Tx und Rx verbunden sind
//  digitalWrite(1, 0);
//  delayMicroseconds(2);
//  if (digitalRead(3)==0){ ;
//  //pinMode(onboardled, OUTPUT);
//  blinken(onboardled, 1000, 200, 3);
//  Einstellen();  // TxD ist mit RxD verbunden Setupmodus (Setup.h)
//  }
// 
//  }
  
  //Fehlerspeicher auslesen 
  ESP.rtcUserMemoryRead(12, wificonnf, sizeof(wificonnf)); 
  ESP.rtcUserMemoryRead(16, httpconnf, sizeof(httpconnf)); 
  ESP.rtcUserMemoryRead(24, &aufzaehler, sizeof(aufzaehler));
  ESP.rtcUserMemoryRead(28, &zuzaehler, sizeof(zuzaehler));
  //Werte in weiteren Variablen übertragen
  wificonnfz[0] = wificonnf[0];
  httpconnfz[0] = httpconnf[0];
  aufzaehlerz = aufzaehler;
  zuzaehlerz = zuzaehler;
    if (zustandalt==1) { // Auf oder Zuzahehler hochzaehlen und Flag setzen ob auf oder zu Bewegung erkannt
    aufzaehler=1;
    zuzaehler=0 ;
    bewflag=1;
    }
    else{
    zuzaehler=1;
    aufzaehler=0;
    bewflag=0;
    }
  digitalWrite(onboardled, HIGH); //Onboardled auschalten 
  
  //Wenn Power OFF + Reset dann Setup ausführen
  if ((resetInfo->reason == 6 and Zyklus[0] == 0)){ //Wenn PowerRes + Zyklus 0
    Zyklus[0]++;
    ESP.rtcUserMemoryWrite(0, Zyklus, sizeof(Zyklus)); // Zyklus Nummer speichern
    blinken(onboardled, 1000, 200, 3);
    Einstellen();
  }

  //OTA Code 
 //debug code
//    debug1 = digitalRead(16);
//    Serial.begin(115200);
//    Serial.println("");
//    Serial.print("ResetInfo ");
//    Serial.println(resetInfo->reason);
//    Serial.print("GPIO16 ");
//    Serial.println(debug1);
  //end debug

  if ((resetInfo->reason == 6 and Zyklus[0] == 1)){ //Wenn PowerRes + Zyklus 1
  Zyklus[0]++;
  ESP.rtcUserMemoryWrite(0, Zyklus, sizeof(Zyklus)); // Zyklus Nummer speichern
  WiFi.forceSleepWake();
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, passwort);
    timeout=0;
    while (WiFi.status() != WL_CONNECTED)
    {
    Serial.print(".");
    blinken(onboardled,50,10,1);
    delay(100);
    timeout++;
    if  (timeout > 100) // Wenn Anmeldung nicht möglich
    
    {
      Serial.println("");
      Serial.println("Netzwerk nicht gefunden");
      delay(10);
      blinken(onboardled,100,50,8);
      system_deep_sleep_set_option(0); 
      system_deep_sleep_instant(0);
      delay(100);
      
    }                           // End timeout
  }
 blinken(onboardled, 4000, 1,1);
 Serial.println("");
 ArduinoOTA.onStart([]() {
 Serial.println("OTA Start");
  });
 ArduinoOTA.onEnd([]() {
 Serial.println("\nEnd");
  });
 ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
 Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
 ArduinoOTA.onError([](ota_error_t error) {
 Serial.printf("Error[%u]: ", error);
   if (error == OTA_AUTH_ERROR) Serial.println("Auth Failed");
   else if (error == OTA_BEGIN_ERROR) Serial.println("Begin Failed");
   else if (error == OTA_CONNECT_ERROR) Serial.println("Connect Failed");
   else if (error == OTA_RECEIVE_ERROR) Serial.println("Receive Failed");
   else if (error == OTA_END_ERROR) Serial.println("End Failed");
 });
 ArduinoOTA.begin();
 Serial.println("Ready for OTA");
 Serial.print("IP address: ");
 Serial.println(WiFi.localIP());
 // blinken(1,1000, 50);
 //goto handleota; //Springe zum OTA handler
 timeout = 0;
 digitalWrite(ffclr, HIGH); //Flip Flop aktivieren
 do{
  ArduinoOTA.handle();
  yield();    
 }while (timeout == 0);
 
 }
//END OTA Code

  //debug code
//    Serial.println("");
//    Serial.print("Zyklus: ");
//    Serial.println(Zyklus[0]);
//    Serial.print("zustandalt: ");
//    Serial.println(!zustandalt);
  //end debug

    
  weiter:

  
  WiFi.forceSleepBegin();       // Sender OFF um Strom zu sparen
  digitalWrite(ffclr, LOW); // Flip Flop R PIN auf LOW, dadurch deaktiv um während Sketch läuuft kein Reset mehr zu erzeugen 
  zaehler.attach_ms(20,entprell); //Ticker aktivieren
  delay(8000); //8 Sekunden lang Schalter überwachen
  if (zustandalt==1) {      // wenn offen lade Zustand vor Sleep um festzustellen ob Zustand immer noch offen //dreh
  ESP.rtcUserMemoryRead(4, zustandsleep, sizeof(zustandsleep)); //Zustand vor letztem Sleep
  }
  else{ zustandsleep[0]=0;} //sonst zurücksetzen auf Zustand zu 
  Zyklus[0]++;   
  ESP.rtcUserMemoryWrite(0, Zyklus, sizeof(Zyklus)); // Zyklus Nummer speichern
  int32_t rssi = wifi_station_get_rssi(); // Signalstärke einlesen
  float ADCValue = 0;
  ADCValue = ESP.getVcc();         //Batteriespannung messen
  String Ubatt = "";
  //Ubatt = String(ADCValue / 1000 - 0.17) + "V"; //  hier Korrektur Wert eintragen

    //----------------- ADC - Kalibrationswerte einlesen -------------------------------------------
 
  char *zeichen;
  zeichen = reinterpret_cast<char*>(&ADCKal34);
  *zeichen = EEPROM.read(240);
  zeichen++;
  *zeichen = EEPROM.read(241);
  zeichen = reinterpret_cast<char*>(&ADCKal24);
  *zeichen = EEPROM.read(242);
  zeichen++;
  *zeichen = EEPROM.read(243);
  if (ADCKal34 > 5000 || ADCKal34 < 2000)ADCKal34 = 3400;
  if (ADCKal24 > 5000 || ADCKal24 < 1500)ADCKal24 = 2400;
  //----------------- ADC Wert korriegieren -------------------------------------------
  unsigned int faktor = ADCKal34 - ADCKal24;
  int Korrekturwert = 2.4 * faktor - ADCKal24;
  float ADCfloat = float(ADCValue + Korrekturwert);
  ADCfloat =ADCfloat/faktor;
  Ubatt = String(ADCfloat, 2) + "V";

  ESP.rtcUserMemoryRead(8, periodenflag1, sizeof(periodenflag1)); // periodenflag1 einlesen 
  //ESP.rtcUserMemoryRead(12, wificonnf, sizeof(wificonnf)); //Wifi Conn Fehlerspeicher auslesen
  //ESP.rtcUserMemoryRead(16, httpconnf, sizeof(httpconnf)); //Http Conn Fehlerspeicher auslesen
  //folgende 4 Zeilen wandeln den wert von Char in String um 
  periodenabstandstring += (char)periodenabstandstr[0]; 
  periodenabstandstring += (char)periodenabstandstr[1]; 
  periodenabstandstring += (char)periodenabstandstr[2]; 
  periodenabstand = periodenabstandstring.toInt(); // und hier in INT damit mann mit dem Wert rechnen kann
    
    //debug code
   // periodenabstand = 1;
    //end debug
  
  if (periodstr[0]== 'J'){ periodenflag1[0] = 1;} //Wenn aus dem EEPROM Periodenflag Info Ja dann setze periodenflag
  else
  {periodenflag1[0] = 0;} 
  WiFi.forceSleepWake();
  WiFi.mode(WIFI_STA);
  //char * ssid = "speedi3";
  WiFi.begin(ssid, passwort);

  for (int i=0; i<=2; i++){
    timeout=0;
    while (WiFi.status() != WL_CONNECTED)
    {
    //Serial.print(".");
    delay(80);
    timeout++;
    if  (timeout > 100) // Wenn Anmeldung nicht möglich
    
    {
      //WiFi.forceSleepBegin();       // Sender OFF um Strom zu sparen
      Serial.println("");
      Serial.println("Netzwerk nicht gefunden");
      delay(10);
      break;
      delay(100);
      
    }                           // End timeout
  }
  
  }

  if (WiFi.status() !=WL_CONNECTED){
      wificonnf[0] ++; //Wifi Connect Fehlerzähler inkrementieren
      ESP.rtcUserMemoryWrite(12, wificonnf, sizeof(wificonnf));
      ESP.rtcUserMemoryWrite(24, &aufzaehler, sizeof(aufzaehler));
      ESP.rtcUserMemoryWrite(28, &zuzaehler, sizeof(zuzaehler));
      system_deep_sleep_set_option(0); 
      system_deep_sleep_instant(0);
  }

   
  wificonnf[0] = 0; //Wifi Connect Fehlerzähler reset
  ESP.rtcUserMemoryWrite(12, wificonnf, sizeof(wificonnf));


 
  setSyncProvider(getNTP_UTCTime1970);
  unsigned long stamp = getNTPTimestamp();
//  Serial.println("");
//  Serial.print("UTC Timestamp=");
//  Serial.println(stamp+946684800UL);
  // UTC
  time_t tT = now();
  //Serial.print ("UTC ");
  //Serial.println(tT);
  int h=hour(tT);
  int mz=minute(tT)/10;
  int me=minute(tT)%10;
  int sz=second(tT)/10;
  int se=second(tT)%10;
  //Serial.printf("%d:%d%d:%d%d Uhr\n", h,mz,me,sz,se);

   // local time
  time_t tTlocal = CE.toLocal(tT);
  //Serial.print ("Local Time ");
  //Serial.println(tTlocal);
  h=hour(tTlocal);
  mz=minute(tTlocal)/10;
  me=minute(tTlocal)%10;
  sz=second(tTlocal)/10;
  se=second(tTlocal)%10;
  short t=day(tTlocal);
  short m=month(tTlocal);
  short j=year(tTlocal);
  //Serial.printf("%d:%d%d:%d%d Uhr\n", h,mz,me,sz,se);
  //Serial.printf("Datum: %d.%d.%d\n",  t,m,j);

 
zaehler.detach();

  if (zustandsleep[0]==2 and zustandalt==1){ //Wenn Zustand vor letztem Sleep auf war und jetziger Zustand auch AUF
    
    if (zuzaehler==0){
    nachricht2=" meldet: immer noch";
    periodenflag1[0]=1;
    }

    else{
    nachricht2=" meldet Bewegung:";
  }
  }  
  
   
  
  else{
    nachricht2=" meldet Bewegung:";
  }
 
  
  
  if (zustandalt==1){ //dreh
    
    zustand=" geoeffnet";
  
  }

  if (zustandalt==0){ //dreh
    zustand=" geschlossen";
  }
  ESP.rtcUserMemoryWrite(24, &aufzaehler, sizeof(aufzaehler));
  ESP.rtcUserMemoryWrite(28, &zuzaehler, sizeof(zuzaehler));
  
  if (aufzaehler >0 and zuzaehler >0){

    if (zustandalt==1){
    zustand ="";
    zustand += String(zuzaehler) + "X geschlossen, " + String(aufzaehler) + "X geoeffnet";
    schlussz = "geoeffnet";
  }

  if (zustandalt==0){
    zustand ="";
    zustand += String(aufzaehler) + "X geoeffnet, " + String(zuzaehler)+ "X geschlossen" ;
    schlussz = "geschlossen";
  }
  }
  else schlussz ="---";
  
  //URL bauen
  url = url + "UBatt=" + Ubatt;      //Parameter Bat Spannung
  url = url + "&WachNr=" + String(Zyklus[0] - 1); //Aufwachzyklus
  url = url + "&Empfang=" + String(rssi); //Parameter Signalstarke
  url = url + "&schlussz=" + urlencode(schlussz); //Parameter letzter Zustand des Kontaktes
  url = url + "&nachricht=" + urlencode(nachricht);
  url = url + "&nachricht2=" + urlencode(nachricht2);
  url = url + "&zustand=" + urlencode(zustand);
  url = url + "&wifif=" + String(wificonnfz[0]);
  url = url + "&httpf=" + String(httpconnfz[0]);
  url = url + "&t=" + String(t);
  url = url + "&m=" + String(m);
  url = url + "&j=" + String(j);
  url = url + "&h=" + String(h);
  url = url + "&mz=" + String(mz) + "&me=" + String(me);
  //url = url + "&me=" + String(me);
  url = url + "&sz=" + String(sz) + "&se=" + String(se);
  //url = url + "&se=" + String(se);

  for (int i=0; i<=2; i++){  //3 x push Versuch starten
  http.begin(url); //HTTP Sende URL
  httpCode= http.GET();
  delay(500);
  if (httpCode == HTTP_CODE_OK) {
      break;
    }
    
    else{
    http.end();
    delay(1500);
    }
  }

  if (httpCode == HTTP_CODE_OK) {
    httpconnf[0] = 0; //http Connect Fehlerzähler reset
    ESP.rtcUserMemoryWrite(16, httpconnf, sizeof(httpconnf));
  }
  if (httpCode != HTTP_CODE_OK) {
      httpconnf[0] ++; //http Connect Fehlerzähler inkrementieren
      ESP.rtcUserMemoryWrite(16, httpconnf, sizeof(httpconnf));
      ESP.rtcUserMemoryWrite(24, &aufzaehler, sizeof(aufzaehler));
      ESP.rtcUserMemoryWrite(28, &zuzaehler, sizeof(zuzaehler));
      system_deep_sleep_set_option(0); 
      system_deep_sleep_instant(0);
    
  }
  http.end();

  if (zustandalt==0 or periodenflag1[0]==0 ){ //wenn Zustand zu oder keine per. Benachrichtigung gewünscht //dreh
  zustandsleep[0]=1;  
  ESP.rtcUserMemoryWrite(4, zustandsleep, sizeof(zustandsleep));
  periodenflag1[0]=0; // periodenflag zurück setzen da Zustand zu oder keine periodische Benachr. gewollt
  ESP.rtcUserMemoryWrite(8, periodenflag1, sizeof(periodenflag1));
  system_deep_sleep_set_option(0); 
  system_deep_sleep_instant(0);
  delay(100);
 }
 else {
  
  if (periodenflag1[0]==1){ // wenn per. Benachrichtigung gewünscht
     //periodenabstand=periodenabstand;
     ESP.rtcUserMemoryWrite(8, periodenflag1, sizeof(periodenflag1));
     }
  else {periodenabstand = 3;}
  zustandsleep[0]=2;   //auf
  ESP.rtcUserMemoryWrite(4, zustandsleep, sizeof(zustandsleep));
  system_deep_sleep_set_option(0);
  system_deep_sleep_instant(periodenabstand*1000000*60);
  delay(100);
 }
}

 void loop() {}





//Sub Interrupt  Change

  void entprell(){
    zustandneu = digitalRead(reedpin); //reedpin zustand einlesen
    zustandneu = !zustandneu; // invertieren für Hatdwareversion mit nur 1 reedpin
    int zustandneu2 = (zustandalt ^ zustandneu); // XOR Verknüpfung der Tasterzustände
    
    if (zustandneu2 == 1){ //wenn Tasterzustand geändert
    prellzaehler ++;       // prellzaehler hochzaehlen
    }
    
    else {                  // sonst prellzaehler reset
      prellzaehler =0;
    }

    if (prellzaehler >=4){  
      if (zustandneu==HIGH){
    aufzaehler++;
    zustandalt = zustandneu;
    
  }
      if (zustandneu==LOW){
    zuzaehler++;
    zustandalt = zustandneu;
  
  }

    }
    
  }
 

//Sub Blinken port = LEDPort  ontime =an Phase in ms offtime= aus Phase in ms wh= Wiederholungen
  void blinken(unsigned int port,unsigned int ontime, unsigned int offtime, unsigned int wiederholungen){

  do{  
  digitalWrite(port, LOW);
  delay(ontime);
  digitalWrite(port, HIGH);
  delay(offtime); 
  wiederholungen--;
  }while (wiederholungen>0);
  
}

 
